@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2010-2021 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
@@ -37,8 +37,7 @@ int knet_host_add(knet_handle_t knet_h, knet_node_id_t host_id)
 	struct knet_host *host = NULL;
 	uint8_t link_idx;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -117,8 +116,7 @@ int knet_host_remove(knet_handle_t knet_h, knet_node_id_t host_id)
 	struct knet_host *host, *removed;
 	uint8_t link_idx;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -188,8 +186,7 @@ int knet_host_set_name(knet_handle_t knet_h, knet_node_id_t host_id, const char
 	int savederrno = 0, err = 0;
 	struct knet_host *host;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -248,8 +245,7 @@ int knet_host_get_name_by_host_id(knet_handle_t knet_h, knet_node_id_t host_id,
 {
 	int savederrno = 0, err = 0;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -287,8 +283,7 @@ int knet_host_get_id_by_host_name(knet_handle_t knet_h, const char *name,
 	int savederrno = 0, err = 0, found = 0;
 	struct knet_host *host;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -333,8 +328,7 @@ int knet_host_get_host_list(knet_handle_t knet_h,
 {
 	int savederrno = 0;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -364,8 +358,7 @@ int knet_host_set_policy(knet_handle_t knet_h, knet_node_id_t host_id,
 	int savederrno = 0, err = 0;
 	uint8_t old_policy;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -414,8 +407,7 @@ int knet_host_get_policy(knet_handle_t knet_h, knet_node_id_t host_id,
 {
 	int savederrno = 0, err = 0;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -454,8 +446,7 @@ int knet_host_get_status(knet_handle_t knet_h, knet_node_id_t host_id,
 	int savederrno = 0, err = 0;
 	struct knet_host *host;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -500,8 +491,7 @@ int knet_host_enable_status_change_notify(knet_handle_t knet_h,
 {
 	int savederrno = 0;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -536,7 +526,7 @@ static void _clear_cbuffers(struct knet_host *host, seq_num_t rx_seq_num)
 
 	memset(host->circular_buffer_defrag, 0, KNET_CBUFFER_SIZE);
 
-	for (i = 0; i < KNET_MAX_LINK; i++) {
+	for (i = 0; i < KNET_DEFRAG_BUFFERS; i++) {
 		memset(&host->defrag_buf[i], 0, sizeof(struct knet_host_defrag_buf));
 	}
 }
@@ -547,12 +537,12 @@ static void _reclaim_old_defrag_bufs(struct knet_host *host, seq_num_t seq_num)
 	int i;
 
 	head = seq_num + 1;
-	tail = seq_num - (KNET_MAX_LINK + 1);
+	tail = seq_num - (KNET_DEFRAG_BUFFERS + 1);
 
 	/*
 	 * expire old defrag buffers
 	 */
-	for (i = 0; i < KNET_MAX_LINK; i++) {
+	for (i = 0; i < KNET_DEFRAG_BUFFERS; i++) {
 		if (host->defrag_buf[i].in_use) {
 			/*
 			 * head has done a rollover to 0+
@@ -583,11 +573,26 @@ int _seq_num_lookup(struct knet_host *host, seq_num_t seq_num, int defrag_buf, i
 	char *dst_cbuf_defrag = host->circular_buffer_defrag;
 	seq_num_t *dst_seq_num = &host->rx_seq_num;
 
+	/*
+	 * There is a potential race condition where the sender
+	 * is overloaded, sending data packets before pings
+	 * can kick in and set the correct dst_seq_num.
+	 *
+	 * if this node is starting up (dst_seq_num = 0),
+	 * it can start rejecing valid packets and get stuck.
+	 *
+	 * Set the dst_seq_num to the first seen packet and
+	 * use that as reference instead.
+	 */
+	if (!*dst_seq_num) {
+		*dst_seq_num = seq_num;
+	}
+
 	if (clear_buf) {
 		_clear_cbuffers(host, seq_num);
 	}
 
-	_reclaim_old_defrag_bufs(host, seq_num);
+	_reclaim_old_defrag_bufs(host, *dst_seq_num);
 
 	if (seq_num < *dst_seq_num) {
 		seq_dist =  (SEQ_MAX - seq_num) + *dst_seq_num;
