@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2012-2021 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
@@ -14,6 +14,7 @@
 #include <string.h>
 #include <pthread.h>
 
+#include "netutils.h"
 #include "internals.h"
 #include "logging.h"
 #include "links.h"
@@ -102,12 +103,11 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 			 struct sockaddr_storage *dst_addr,
 			 uint64_t flags)
 {
-	int savederrno = 0, err = 0, i;
-	struct knet_host *host;
+	int savederrno = 0, err = 0, i, wipelink = 0, link_idx;
+	struct knet_host *host, *tmp_host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -192,7 +192,13 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		goto exit_unlock;
 	}
 
-	memmove(&link->src_addr, src_addr, sizeof(struct sockaddr_storage));
+	/*
+	 * errors happening after this point should trigger
+	 * a memset of the link
+	 */
+	wipelink = 1;
+
+	copy_sockaddr(&link->src_addr, src_addr);
 
 	err = knet_addrtostr(src_addr, sizeof(struct sockaddr_storage),
 			     link->status.src_ipaddr, KNET_MAX_HOST_LEN,
@@ -219,7 +225,7 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 
 		link->dynamic = KNET_LINK_STATIC;
 
-		memmove(&link->dst_addr, dst_addr, sizeof(struct sockaddr_storage));
+		copy_sockaddr(&link->dst_addr, dst_addr);
 		err = knet_addrtostr(dst_addr, sizeof(struct sockaddr_storage),
 				     link->status.dst_ipaddr, KNET_MAX_HOST_LEN,
 				     link->status.dst_port, KNET_MAX_PORT_LEN);
@@ -251,6 +257,28 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	link->latency_cur_samples = 0;
 	link->flags = flags;
 
+	/*
+	 * check for DYNIP vs STATIC collisions.
+	 * example: link0 is static, user attempts to configure link1 as dynamic with the same source
+	 * address/port.
+	 * This configuration is invalid and would cause ACL collisions.
+	 */
+	for (tmp_host = knet_h->host_head; tmp_host != NULL; tmp_host = tmp_host->next) {
+		for (link_idx = 0; link_idx < KNET_MAX_LINK; link_idx++) {
+			if (&tmp_host->link[link_idx] == link)
+				continue;
+
+			if ((!memcmp(&tmp_host->link[link_idx].src_addr, &link->src_addr, sizeof(struct sockaddr_storage))) &&
+			    (tmp_host->link[link_idx].dynamic != link->dynamic)) {
+				savederrno = EINVAL;
+				err = -1;
+				log_err(knet_h, KNET_SUB_LINK, "Failed to configure host %u link %u dyn %u. Conflicts with host %u link %u dyn %u: %s",
+					host_id, link_id, link->dynamic, tmp_host->host_id, link_idx, tmp_host->link[link_idx].dynamic, strerror(savederrno));
+				goto exit_unlock;
+			}
+		}
+	}
+
 	savederrno = pthread_mutex_init(&link->link_stats_mutex, NULL);
 	if (savederrno) {
 		log_err(knet_h, KNET_SUB_LINK, "Unable to initialize link stats mutex: %s", strerror(savederrno));
@@ -261,7 +289,7 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	if (transport_link_set_config(knet_h, link, transport) < 0) {
 		savederrno = errno;
 		err = -1;
-		goto exit_unlock;
+		goto exit_transport_err;
 	}
 
 	/*
@@ -273,16 +301,19 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	    (link->dynamic == KNET_LINK_STATIC)) {
 		log_debug(knet_h, KNET_SUB_LINK, "Configuring default access lists for host: %u link: %u socket: %d",
 			  host_id, link_id, link->outsock);
-		if ((check_add(knet_h, link->outsock, transport, -1,
+		if ((check_add(knet_h, link, -1,
 			       &link->dst_addr, &link->dst_addr,
 			       CHECK_TYPE_ADDRESS, CHECK_ACCEPT) < 0) && (errno != EEXIST)) {
 			log_warn(knet_h, KNET_SUB_LINK, "Failed to configure default access lists for host: %u link: %u", host_id, link_id);
 			savederrno = errno;
 			err = -1;
-			goto exit_unlock;
+			goto exit_acl_error;
 		}
 	}
 
+	/*
+	 * no errors should happen after link is configured
+	 */
 	link->configured = 1;
 	log_debug(knet_h, KNET_SUB_LINK, "host: %u link: %u is configured",
 		  host_id, link_id);
@@ -322,7 +353,33 @@ int knet_link_set_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		link->has_valid_mtu = 1;
 	}
 
+exit_acl_error:
+	/*
+	 * if creating access lists has error, we only need to clean
+	 * the transport and the stuff below.
+	 */
+	if (err < 0) {
+		if ((transport_link_clear_config(knet_h, link) < 0)  &&
+		    (errno != EBUSY)) {
+			log_warn(knet_h, KNET_SUB_LINK, "Failed to deconfigure transport for host %u link %u: %s", host_id, link_id, strerror(errno));
+		}
+	}
+exit_transport_err:
+	/*
+	 * if transport has errors, transport will clean after itself
+	 * and we only need to clean the mutex
+	 */
+	if (err < 0) {
+		pthread_mutex_destroy(&link->link_stats_mutex);
+	}
 exit_unlock:
+	/*
+	 * re-init the link on error
+	 */
+	if ((err < 0) && (wipelink)) {
+		memset(link, 0, sizeof(struct knet_link));
+		link->link_id = link_id;
+	}
 	pthread_rwlock_unlock(&knet_h->global_rwlock);
 	errno = err ? savederrno : 0;
 	return err;
@@ -339,8 +396,7 @@ int knet_link_get_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -428,8 +484,7 @@ int knet_link_clear_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 	int sock;
 	uint8_t transport;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -481,7 +536,7 @@ int knet_link_clear_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 	 */
 	if ((transport_get_acl_type(knet_h, link->transport) == USE_GENERIC_ACL) &&
 	    (link->dynamic == KNET_LINK_STATIC)) {
-		if ((check_rm(knet_h, link->outsock, link->transport,
+		if ((check_rm(knet_h, link,
 			      &link->dst_addr, &link->dst_addr,
 			      CHECK_TYPE_ADDRESS, CHECK_ACCEPT) < 0) && (errno != ENOENT)) {
 			err = -1;
@@ -510,9 +565,9 @@ int knet_link_clear_config(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 	 * remove any other access lists when the socket is no
 	 * longer in use by the transport.
 	 */
-	if ((transport_get_acl_type(knet_h, link->transport) == USE_GENERIC_ACL) &&
+	if ((transport_get_acl_type(knet_h, transport) == USE_GENERIC_ACL) &&
 	    (knet_h->knet_transport_fd_tracker[sock].transport == KNET_MAX_TRANSPORTS)) {
-		check_rmall(knet_h, sock, transport);
+		check_rmall(knet_h, link);
 	}
 
 	pthread_mutex_destroy(&link->link_stats_mutex);
@@ -543,8 +598,7 @@ int knet_link_set_enable(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -613,8 +667,7 @@ int knet_link_get_enable(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -670,8 +723,7 @@ int knet_link_set_pong_count(knet_handle_t knet_h, knet_node_id_t host_id, uint8
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -731,8 +783,7 @@ int knet_link_get_pong_count(knet_handle_t knet_h, knet_node_id_t host_id, uint8
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -788,8 +839,7 @@ int knet_link_set_ping_timers(knet_handle_t knet_h, knet_node_id_t host_id, uint
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -861,8 +911,7 @@ int knet_link_get_ping_timers(knet_handle_t knet_h, knet_node_id_t host_id, uint
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -931,8 +980,7 @@ int knet_link_set_priority(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 	struct knet_link *link;
 	uint8_t old_priority;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -1004,8 +1052,7 @@ int knet_link_get_priority(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -1061,8 +1108,7 @@ int knet_link_get_link_list(knet_handle_t knet_h, knet_node_id_t host_id,
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -1117,8 +1163,7 @@ int knet_link_get_status(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -1212,100 +1257,6 @@ exit_unlock:
 	return err;
 }
 
-int knet_link_add_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
-		      struct sockaddr_storage *ss1,
-		      struct sockaddr_storage *ss2,
-		      check_type_t type, check_acceptreject_t acceptreject)
-{
-	int savederrno = 0, err = 0;
-	struct knet_host *host;
-	struct knet_link *link;
-
-	if (!knet_h) {
-		errno = EINVAL;
-		return -1;
-	}
-
-	if (!ss1) {
-		errno = EINVAL;
-		return -1;
-	}
-
-	if ((type != CHECK_TYPE_ADDRESS) &&
-	    (type != CHECK_TYPE_MASK) &&
-	    (type != CHECK_TYPE_RANGE)) {
-		errno = EINVAL;
-		return -1;
-	}
-
-	if ((acceptreject != CHECK_ACCEPT) &&
-	    (acceptreject != CHECK_REJECT)) {
-		errno = EINVAL;
-		return -1;
-	}
-
-	if ((type != CHECK_TYPE_ADDRESS) && (!ss2)) {
-		errno = EINVAL;
-		return -1;
-	}
-
-	if ((type == CHECK_TYPE_RANGE) &&
-	    (ss1->ss_family != ss2->ss_family)) {
-			errno = EINVAL;
-			return -1;
-	}
-
-	if (link_id >= KNET_MAX_LINK) {
-		errno = EINVAL;
-		return -1;
-	}
-
-	savederrno = get_global_wrlock(knet_h);
-	if (savederrno) {
-		log_err(knet_h, KNET_SUB_LINK, "Unable to get write lock: %s",
-			strerror(savederrno));
-		errno = savederrno;
-		return -1;
-	}
-
-	host = knet_h->host_index[host_id];
-	if (!host) {
-		err = -1;
-		savederrno = EINVAL;
-		log_err(knet_h, KNET_SUB_LINK, "Unable to find host %u: %s",
-			host_id, strerror(savederrno));
-		goto exit_unlock;
-	}
-
-	link = &host->link[link_id];
-
-	if (!link->configured) {
-		err = -1;
-		savederrno = EINVAL;
-		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is not configured: %s",
-			host_id, link_id, strerror(savederrno));
-		goto exit_unlock;
-	}
-
-	if (link->dynamic != KNET_LINK_DYNIP) {
-		err = -1;
-		savederrno = EINVAL;
-		log_err(knet_h, KNET_SUB_LINK, "host %u link %u is a point to point connection: %s",
-			host_id, link_id, strerror(savederrno));
-		goto exit_unlock;
-	}
-
-	err = check_add(knet_h, transport_link_get_acl_fd(knet_h, link), link->transport, -1,
-			ss1, ss2, type, acceptreject);
-	savederrno = errno;
-
-exit_unlock:
-	pthread_rwlock_unlock(&knet_h->global_rwlock);
-
-	errno = savederrno;
-	return err;
-}
-
 int knet_link_insert_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
 			 int index,
 			 struct sockaddr_storage *ss1,
@@ -1316,8 +1267,7 @@ int knet_link_insert_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -1390,7 +1340,7 @@ int knet_link_insert_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t l
 		goto exit_unlock;
 	}
 
-	err = check_add(knet_h, transport_link_get_acl_fd(knet_h, link), link->transport, index,
+	err = check_add(knet_h, link, index,
 			ss1, ss2, type, acceptreject);
 	savederrno = errno;
 
@@ -1401,6 +1351,15 @@ exit_unlock:
 	return err;
 }
 
+int knet_link_add_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
+		      struct sockaddr_storage *ss1,
+		      struct sockaddr_storage *ss2,
+		      check_type_t type, check_acceptreject_t acceptreject)
+{
+	return knet_link_insert_acl(knet_h, host_id, link_id, -1, ss1, ss2, type, acceptreject);
+}
+
+
 int knet_link_rm_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_id,
 		     struct sockaddr_storage *ss1,
 		     struct sockaddr_storage *ss2,
@@ -1410,8 +1369,7 @@ int knet_link_rm_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -1484,7 +1442,7 @@ int knet_link_rm_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t link_
 		goto exit_unlock;
 	}
 
-	err = check_rm(knet_h, transport_link_get_acl_fd(knet_h, link), link->transport,
+	err = check_rm(knet_h, link,
 		       ss1, ss2, type, acceptreject);
 	savederrno = errno;
 
@@ -1501,8 +1459,7 @@ int knet_link_clear_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t li
 	struct knet_host *host;
 	struct knet_link *link;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -1546,7 +1503,7 @@ int knet_link_clear_acl(knet_handle_t knet_h, knet_node_id_t host_id, uint8_t li
 		goto exit_unlock;
 	}
 
-	check_rmall(knet_h, transport_link_get_acl_fd(knet_h, link), link->transport);
+	check_rmall(knet_h, link);
 
 exit_unlock:
 	pthread_rwlock_unlock(&knet_h->global_rwlock);
