@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2021 Red Hat, Inc.  All rights reserved.
  *
  * Author: Christine Caulfield <ccaulfie@redhat.com>
  *
@@ -27,13 +27,18 @@ typedef struct {
 	void (*protocheck_rmall)	(void *fd_tracker_match_entry_head);
 } check_ops_t;
 
-int check_add(knet_handle_t knet_h, int sock, uint8_t transport, int index,
+int check_add(knet_handle_t knet_h, struct knet_link *kn_link,
+	      int index,
 	      struct sockaddr_storage *ss1, struct sockaddr_storage *ss2,
 	      check_type_t type, check_acceptreject_t acceptreject);
-int check_rm(knet_handle_t knet_h, int sock, uint8_t transport,
+
+int check_rm(knet_handle_t knet_h, struct knet_link *kn_link,
 	     struct sockaddr_storage *ss1, struct sockaddr_storage *ss2,
 	     check_type_t type, check_acceptreject_t acceptreject);
-void check_rmall(knet_handle_t knet_h, int sock, uint8_t transport);
-int check_validate(knet_handle_t knet_h, int sock, uint8_t transport, struct sockaddr_storage *checkip);
+
+void check_rmall(knet_handle_t knet_h, struct knet_link *kn_link);
+
+int check_validate(knet_handle_t knet_h, struct knet_link *kn_link,
+		   struct sockaddr_storage *checkip);
 
 #endif
