@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2010-2021 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
@@ -46,6 +46,13 @@ socklen_t sockaddr_len(const struct sockaddr_storage *ss)
 	}
 }
 
+/* Only copy the valid parts of a sockaddr* */
+void copy_sockaddr(struct sockaddr_storage *sout, const struct sockaddr_storage *sin)
+{
+	memset(sout, 0, sizeof(struct sockaddr_storage));
+	memmove(sout, sin, sockaddr_len(sin));
+}
+
 /*
  * exported APIs
  */
