@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2021 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
@@ -26,7 +26,7 @@ static void test(void)
 
 	setup_logpipes(logfds);
 
-	printf("Test knet_handle_free with invalid knet_h\n");
+	printf("Test knet_handle_free with invalid knet_h (part 1)\n");
 
 	if ((!knet_handle_free(NULL)) || (errno != EINVAL)) {
 		printf("knet_handle_free failed to detect invalid parameter\n");
@@ -62,6 +62,15 @@ static void test(void)
 		exit(FAIL);
 	}
 
+	printf("Test knet_handle_free with invalid knet_h (part 2)\n");
+
+	if ((!knet_handle_free(knet_h + 1)) || (errno != EINVAL)) {
+		printf("knet_handle_free failed to detect invalid parameter\n");
+		flush_logs(logfds[0], stdout);
+		close_logpipes(logfds);
+		exit(FAIL);
+	}
+
 	if (knet_handle_free(knet_h) < 0) {
 		printf("knet_handle_free failed: %s\n", strerror(errno));
 		flush_logs(logfds[0], stdout);
