@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2019-2021 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
@@ -200,7 +200,7 @@ static void test(void)
 	host = knet_h->host_index[1];
 	link = &host->link[0];
 
-	if (knet_h->knet_transport_fd_tracker[link->outsock].access_list_match_entry_head) {
+	if (link->access_list_match_entry_head) {
 		printf("match list not empty!");
 		knet_link_clear_config(knet_h, 1, 0);
 		knet_host_remove(knet_h, 1);
@@ -220,7 +220,7 @@ static void test(void)
 		exit(FAIL);
 	}
 
-	if (!knet_h->knet_transport_fd_tracker[link->outsock].access_list_match_entry_head) {
+	if (!link->access_list_match_entry_head) {
 		printf("match list empty!");
 		knet_link_clear_config(knet_h, 1, 0);
 		knet_host_remove(knet_h, 1);
