@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2021 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *
@@ -178,6 +178,26 @@ static void test(void)
 
 	flush_logs(logfds[0], stdout);
 
+	printf("Test knet_send_sync with no filter configured\n");
+
+	channel = 1;
+	if ((!knet_send_sync(knet_h, send_buff, KNET_MAX_PACKET_SIZE, channel)) || (errno != ENETDOWN)) {
+		printf("knet_send_sync Did not return ENETDOWN for no filter installed: %s\n", strerror(errno));
+		knet_handle_free(knet_h);
+		flush_logs(logfds[0], stdout);
+		close_logpipes(logfds);
+		exit(FAIL);
+	}
+
+	if (knet_handle_enable_filter(knet_h, NULL, dhost_filter) < 0) {
+		printf("knet_handle_enable_filter failed: %s\n", strerror(errno));
+		knet_handle_free(knet_h);
+		flush_logs(logfds[0], stdout);
+		close_logpipes(logfds);
+		exit(FAIL);
+	}
+
+
 	printf("Test knet_send_sync with unconfigured channel\n");
 
 	channel = 0;
@@ -233,14 +253,6 @@ static void test(void)
 		exit(FAIL);
 	}
 
-	if (knet_handle_enable_filter(knet_h, NULL, dhost_filter) < 0) {
-		printf("knet_handle_enable_filter failed: %s\n", strerror(errno));
-		knet_handle_free(knet_h);
-		flush_logs(logfds[0], stdout);
-		close_logpipes(logfds);
-		exit(FAIL);
-	}
-
 	dhost_filter_ret = -1;
 
 	if ((knet_send_sync(knet_h, send_buff, KNET_MAX_PACKET_SIZE, channel) == sizeof(send_buff)) || (errno != EFAULT)) {
