@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2015-2021 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
@@ -99,7 +99,8 @@ static void _handle_check_each(knet_handle_t knet_h, struct knet_host *dst_host,
 retry:
 		if (transport_get_connection_oriented(knet_h, dst_link->transport) == TRANSPORT_PROTO_NOT_CONNECTION_ORIENTED) {
 			len = sendto(dst_link->outsock, outbuf, outlen,	MSG_DONTWAIT | MSG_NOSIGNAL,
-				     (struct sockaddr *) &dst_link->dst_addr, sizeof(struct sockaddr_storage));
+				     (struct sockaddr *) &dst_link->dst_addr,
+				     knet_h->knet_transport_fd_tracker[dst_link->outsock].sockaddr_len);
 		} else {
 			len = sendto(dst_link->outsock, outbuf, outlen,	MSG_DONTWAIT | MSG_NOSIGNAL, NULL, 0);
 		}
