@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2020 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2012-2021 Red Hat, Inc.  All rights reserved.
  *
  * Authors: Fabio M. Di Nitto <fabbione@kronosnet.org>
  *          Federico Simoncelli <fsimon@kronosnet.org>
@@ -63,6 +63,7 @@ static int _dispatch_to_links(knet_handle_t knet_h, struct knet_host *dst_host,
 		msg_idx = 0;
 		while (msg_idx < msgs_to_send) {
 			msg[msg_idx].msg_hdr.msg_name = &cur_link->dst_addr;
+			msg[msg_idx].msg_hdr.msg_namelen = knet_h->knet_transport_fd_tracker[cur_link->outsock].sockaddr_len;
 
 			/* Cast for Linux/BSD compatibility */
 			for (i=0; i<(unsigned int)msg[msg_idx].msg_hdr.msg_iovlen; i++) {
@@ -178,6 +179,9 @@ static int _parse_recv_from_sock(knet_handle_t knet_h, size_t inlen, int8_t chan
 		goto out_unlock;
 	}
 
+	memset(dst_host_ids_temp, 0, sizeof(dst_host_ids_temp));
+	memset(dst_host_ids, 0, sizeof(dst_host_ids));
+
 	/*
 	 * move this into a separate function to expand on
 	 * extra switching rules
@@ -553,7 +557,7 @@ static int _parse_recv_from_sock(knet_handle_t knet_h, size_t inlen, int8_t chan
 	msg_idx = 0;
 
 	while (msg_idx < msgs_to_send) {
-		msg[msg_idx].msg_hdr.msg_namelen = sizeof(struct sockaddr_storage);
+		msg[msg_idx].msg_hdr.msg_namelen = sizeof(struct sockaddr_storage); /* this will set properly in _dispatch_to_links() */
 		msg[msg_idx].msg_hdr.msg_iov = &iov_out[msg_idx][0];
 		msg[msg_idx].msg_hdr.msg_iovlen = iovcnt_out;
 		msg_idx++;
@@ -591,6 +595,11 @@ static void _handle_send_to_links(knet_handle_t knet_h, struct msghdr *msg, int
 	ssize_t inlen = 0;
 	int savederrno = 0, docallback = 0;
 
+	/*
+	 * make sure BSD gets the right size
+	 */
+	msg->msg_namelen = knet_h->knet_transport_fd_tracker[sockfd].sockaddr_len;
+
 	if ((channel >= 0) &&
 	    (channel < KNET_DATAFD_MAX) &&
 	    (!knet_h->sockfd[channel].is_socket)) {
@@ -749,8 +758,7 @@ int knet_send_sync(knet_handle_t knet_h, const char *buff, const size_t buff_len
 {
 	int savederrno = 0, err = 0;
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
@@ -787,6 +795,12 @@ int knet_send_sync(knet_handle_t knet_h, const char *buff, const size_t buff_len
 		return -1;
 	}
 
+	if (!knet_h->dst_host_filter_fn) {
+		savederrno = ENETDOWN;
+		err = -1;
+		goto out;
+	}
+
 	if (!knet_h->sockfd[channel].in_use) {
 		savederrno = EINVAL;
 		err = -1;
@@ -821,8 +835,7 @@ ssize_t knet_send(knet_handle_t knet_h, const char *buff, const size_t buff_len,
 	ssize_t err = 0;
 	struct iovec iov_out[1];
 
-	if (!knet_h) {
-		errno = EINVAL;
+	if (!_is_valid_handle(knet_h)) {
 		return -1;
 	}
 
