@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2019 Red Hat, Inc.  All rights reserved.
+ * Copyright (C) 2016-2021 Red Hat, Inc.  All rights reserved.
  *
  * Author: Christine Caulfield <ccaulfie@redhat.com>
  *
@@ -27,16 +27,18 @@
 #include "link.h"
 #include "logging.h"
 #include "common.h"
+#include "netutils.h"
 #include "transport_common.h"
 #include "transport_udp.h"
+#include "transports.h"
 #include "threads_common.h"
 
 typedef struct udp_handle_info {
-	struct knet_list_head links_list;
+	struct qb_list_head links_list;
 } udp_handle_info_t;
 
 typedef struct udp_link_info {
-	struct knet_list_head list;
+	struct qb_list_head list;
 	struct sockaddr_storage local_address;
 	int socket_fd;
 	int on_epoll;
@@ -56,7 +58,7 @@ int udp_transport_link_set_config(knet_handle_t knet_h, struct knet_link *kn_lin
 	/*
 	 * Only allocate a new link if the local address is different
 	 */
-	knet_list_for_each_entry(info, &handle_info->links_list, list) {
+	qb_list_for_each_entry(info, &handle_info->links_list, list) {
 		if (memcmp(&info->local_address, &kn_link->src_addr, sizeof(struct sockaddr_storage)) == 0) {
 			log_debug(knet_h, KNET_SUB_TRANSP_UDP, "Re-using existing UDP socket for new link");
 			kn_link->outsock = info->socket_fd;
@@ -141,7 +143,7 @@ int udp_transport_link_set_config(knet_handle_t knet_h, struct knet_link *kn_lin
 
 	info->on_epoll = 1;
 
-	if (_set_fd_tracker(knet_h, sock, KNET_TRANSPORT_UDP, 0, info) < 0) {
+	if (_set_fd_tracker(knet_h, sock, KNET_TRANSPORT_UDP, 0, sockaddr_len(&kn_link->src_addr), info) < 0) {
 		savederrno = errno;
 		err = -1;
 		log_err(knet_h, KNET_SUB_TRANSP_UDP, "Unable to set fd tracker: %s",
@@ -151,7 +153,7 @@ int udp_transport_link_set_config(knet_handle_t knet_h, struct knet_link *kn_lin
 
 	memmove(&info->local_address, &kn_link->src_addr, sizeof(struct sockaddr_storage));
 	info->socket_fd = sock;
-	knet_list_add(&info->list, &handle_info->links_list);
+	qb_list_add(&info->list, &handle_info->links_list);
 
 	kn_link->outsock = sock;
 	kn_link->transport_link = info;
@@ -216,7 +218,7 @@ int udp_transport_link_clear_config(knet_handle_t knet_h, struct knet_link *kn_l
 		info->on_epoll = 0;
 	}
 
-	if (_set_fd_tracker(knet_h, info->socket_fd, KNET_MAX_TRANSPORTS, 0, NULL) < 0) {
+	if (_set_fd_tracker(knet_h, info->socket_fd, KNET_MAX_TRANSPORTS, 0, sockaddr_len(&kn_link->src_addr), NULL) < 0) {
 		savederrno = errno;
 		err = -1;
 		log_err(knet_h, KNET_SUB_TRANSP_UDP, "Unable to set fd tracker: %s",
@@ -225,7 +227,7 @@ int udp_transport_link_clear_config(knet_handle_t knet_h, struct knet_link *kn_l
 	}
 
 	close(info->socket_fd);
-	knet_list_del(&info->list);
+	qb_list_del(&info->list);
 	free(kn_link->transport_link);
 
 exit_error:
@@ -247,7 +249,7 @@ int udp_transport_free(knet_handle_t knet_h)
 	/*
 	 * keep it here while we debug list usage and such
 	 */
-	if (!knet_list_empty(&handle_info->links_list)) {
+	if (!qb_list_empty(&handle_info->links_list)) {
 		log_err(knet_h, KNET_SUB_TRANSP_UDP, "Internal error. handle list is not empty");
 		return -1;
 	}
@@ -277,7 +279,7 @@ int udp_transport_init(knet_handle_t knet_h)
 
 	knet_h->transports[KNET_TRANSPORT_UDP] = handle_info;
 
-	knet_list_init(&handle_info->links_list);
+	qb_list_init(&handle_info->links_list);
 
 	return 0;
 }
@@ -362,6 +364,46 @@ static int read_errs_from_sock(knet_handle_t knet_h, int sockfd)
 									log_debug(knet_h, KNET_SUB_TRANSP_UDP, "Received ICMP error from %s: %s destination unknown", addr_str, strerror(sock_err->ee_errno));
 								} else {
 									log_debug(knet_h, KNET_SUB_TRANSP_UDP, "Received ICMP error from %s: %s %s", addr_str, strerror(sock_err->ee_errno), addr_remote_str);
+									if ((sock_err->ee_errno == ECONNREFUSED) || /* knet is not running on the other node */
+									    (sock_err->ee_errno == ECONNABORTED) || /* local kernel closed the socket */
+									    (sock_err->ee_errno == ENONET)       || /* network does not exist */
+									    (sock_err->ee_errno == ENETUNREACH)  || /* network unreachable */
+									    (sock_err->ee_errno == EHOSTUNREACH) || /* host unreachable */
+									    (sock_err->ee_errno == EHOSTDOWN)    || /* host down (from kernel/net/ipv4/icmp.c */
+									    (sock_err->ee_errno == ENETDOWN)) {     /* network down */
+										struct knet_host *host = NULL;
+										struct knet_link *kn_link = NULL;
+										int link_idx, found = 0;
+
+										for (host = knet_h->host_head; host != NULL; host = host->next) {
+											for (link_idx = 0; link_idx < KNET_MAX_LINK; link_idx++) {
+												kn_link = &host->link[link_idx];
+												if (kn_link->outsock == sockfd) {
+													if (!cmpaddr(&remote, &kn_link->dst_addr)) {
+														found = 1;
+														break;
+													}
+												}
+											}
+											if (found) {
+												break;
+											}
+										}
+
+										if ((host) && (kn_link) &&
+										    (kn_link->status.connected)) {
+											log_debug(knet_h, KNET_SUB_TRANSP_UDP, "Setting down host %u link %i", host->host_id, kn_link->link_id);
+											/*
+											 * setting transport_connected = 0 will trigger
+											 * thread_heartbeat link_down process.
+											 *
+											 * the process terminates calling into transport_link_down
+											 * below that will set transport_connected = 1
+											 */
+											kn_link->transport_connected = 0;
+										}
+
+									}
 								}
 							}
 							break;
@@ -395,7 +437,13 @@ int udp_transport_tx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err,
 			read_errs_from_sock(knet_h, sockfd);
 			return 0;
 		}
-		if (recv_errno == EINVAL || recv_errno == EPERM) {
+		if ((recv_errno == EINVAL) || (recv_errno == EPERM) ||
+		    (recv_errno == ENETUNREACH) || (recv_errno == ENETDOWN)) {
+#ifdef DEBUG
+			if ((recv_errno == ENETUNREACH) || (recv_errno == ENETDOWN)) {
+				log_debug(knet_h, KNET_SUB_TRANSP_UDP, "Sock: %d is unreachable.", sockfd);
+			}
+#endif
 			return -1;
 		}
 		if ((recv_errno == ENOBUFS) || (recv_errno == EAGAIN)) {
@@ -415,9 +463,9 @@ int udp_transport_tx_sock_error(knet_handle_t knet_h, int sockfd, int recv_err,
 int udp_transport_rx_is_data(knet_handle_t knet_h, int sockfd, struct knet_mmsghdr *msg)
 {
 	if (msg->msg_len == 0)
-		return 0;
+		return KNET_TRANSPORT_RX_NOT_DATA_CONTINUE;
 
-	return 2;
+	return KNET_TRANSPORT_RX_IS_DATA;
 }
 
 int udp_transport_link_dyn_connect(knet_handle_t knet_h, int sockfd, struct knet_link *kn_link)
@@ -426,7 +474,11 @@ int udp_transport_link_dyn_connect(knet_handle_t knet_h, int sockfd, struct knet
 	return 0;
 }
 
-int udp_transport_link_get_acl_fd(knet_handle_t knet_h, struct knet_link *kn_link)
+int udp_transport_link_is_down(knet_handle_t knet_h, struct knet_link *kn_link)
 {
-	return kn_link->outsock;
+	/*
+	 * see comments about handling ICMP error messages
+	 */
+	kn_link->transport_connected = 1;
+	return 0;
 }
